==> role_hierarchy-8.x-2.1.tar.gz: role_hierarchy/src/Service/RoleHierarchyHelper.php <==
<?php

namespace Drupal\role_hierarchy\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;

/**
 * Class RoleHierarchyHelper provides core functions for role_hierarchy.
 *
 * @package Drupal\role_hierarchy
 */
class RoleHierarchyHelper {

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->settings = $configFactory->get('role_hierarchy.settings');
  }

  /**
   * Get the lowest/highest user role weight, depending on settings.
   */
  public function getUserRoleWeight(User $user) {
    return $this->getAccountRoleWeight($user);
  }

  /**
   * Get the lowest/highest account role, depending on settings.
   */
  public function getAccountRoleWeight(AccountInterface $account) {
    $account_roles = Role::loadMultiple(array_values($account->getRoles()));

    if (!empty($this->settings->get('invert'))) {
      $result = max(array_map([__CLASS__, 'getRoleWeight'], $account_roles));
    }
    else {
      $result = min(array_map([__CLASS__, 'getRoleWeight'], $account_roles));
    }
    return $result;
  }

  /**
   * Get the weight of a role as configured in /admin/people/roles.
   *
   * @param Role|string $role
   *   The user role.
   *
   * @return int
   *   The weight of the role.
   */
  public function getRoleWeight($role) {
    if ($role == 'workflow_author') {
      return 9999;
    }

    if (is_string($role)) {
      $role = Role::load($role);
    }

    return $role->getWeight();
  }

  /**
   * Check if an user can edit another user by comparing their weights.
   *
   * @param $user_role_weight
   *   The user role weight.
   * @param $edited_role_weight
   *   The edited user role weight.
   *
   * @return bool
   */
  public function hasRoleEditAccess($user_role_weight, $edited_role_weight) {
    if (!empty($this->settings->get('invert'))) {
      if (!empty($this->settings->get('strict'))) {
        return $user_role_weight > $edited_role_weight;
      }
      else {
        return $user_role_weight >= $edited_role_weight;
      }
    }
    else {
      if (!empty($this->settings->get('strict'))) {
        return $user_role_weight < $edited_role_weight;
      }
      else {
        return $user_role_weight <= $edited_role_weight;
      }
    }
  }

  /**
   * Determines if an account has edit access on an user.
   *
   * @param AccountInterface $account
   *   The account that tries to edit.
   *
   * @param User $user
   *   The user.
   *
   * @return bool
   */
  public function hasEditAccess(AccountInterface $account, User $user) {
    if ($account->id() == 1) {
      return TRUE;
    }

    if ($account->id() == $user->id()) {
      return TRUE;
    }

    if ($user->id() == 1) {
      return FALSE;
    }
    $account_weight = $this->getAccountRoleWeight($account);
    $user_weight = $this->getUserRoleWeight($user);
    return $this->hasRoleEditAccess($account_weight, $user_weight);
  }

}

==> role_hierarchy-8.x-2.2.tar.gz: role_hierarchy/src/Service/RoleHierarchyHelper.php <==
<?php

namespace Drupal\role_hierarchy\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;

/**
 * Class RoleHierarchyHelper provides core functions for role_hierarchy.
 *
 * @package Drupal\role_hierarchy
 */
class RoleHierarchyHelper {

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   *
   * The module settings.
   */
  protected $settings;

  /**
   * RoleHierarchyHelper constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->settings = $configFactory->get('role_hierarchy.settings');
  }

  /**
   * Get the lowest/highest user role, depending on settings.
   *
   * @param \Drupal\user\UserInterface $user
   *   The current user.
   *
   * @return int
   *   The user  role weight.
   */
  public function getUserRoleWeight(UserInterface $user) {
    return $this->getAccountRoleWeight($user);
  }

  /**
   * Get the lowest/highest account role, depending on settings.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current account.
   *
   * @return int
   *   The account role weight.
   */
  public function getAccountRoleWeight(AccountInterface $account) {
    $account_roles = Role::loadMultiple(array_values($account->getRoles()));
    $non_hierarchical_roles = $this->settings->get('non_hierarchical_roles') ?? [];
    $account_roles = array_filter($account_roles, function ($role) use ($non_hierarchical_roles) {
      return empty($non_hierarchical_roles[$role->id()]);
    });

    if (!empty($this->settings->get('invert'))) {
      if (empty($account_roles)) {
        return -9999;
      }
      $result = max(array_map([__CLASS__, 'getRoleWeight'], $account_roles));
    }
    else {
      if (empty($account_roles)) {
        return 9999;
      }
      $result = min(array_map([__CLASS__, 'getRoleWeight'], $account_roles));
    }
    return $result;
  }

  /**
   * Get the weight of a role as configured in /admin/people/roles.
   *
   * @param Role|string $role
   *   The user role.
   *
   * @return int
   *   The weight of the role.
   */
  public function getRoleWeight($role) {
    if ($role == 'workflow_author') {
      return 9999;
    }

    if (is_string($role)) {
      $role = Role::load($role);
    }

    return $role->getWeight();
  }

  /**
   * Check if an user can edit another user by comparing their weights.
   *
   * @param $user_role_weight
   *   The user role weight.
   * @param $edited_role_weight
   *   The edited user role weight.
   *
   * @return bool
   */
  public function hasRoleEditAccess($user_role_weight, $edited_role_weight) {
    if (!empty($this->settings->get('invert'))) {
      if (!empty($this->settings->get('strict'))) {
        return $user_role_weight > $edited_role_weight;
      }
      else {
        return $user_role_weight >= $edited_role_weight;
      }
    }
    else {
      if (!empty($this->settings->get('strict'))) {
        return $user_role_weight < $edited_role_weight;
      }
      else {
        return $user_role_weight <= $edited_role_weight;
      }
    }
  }

  /**
   * Determines if an account has edit access on an user.
   *
   * @param AccountInterface $account
   *   The account that tries to edit.
   *
   * @param User $user
   *   The user.
   *
   * @return bool
   */
  public function hasEditAccess(AccountInterface $account, User $user) {
    if ($account->id() == 1) {
      return TRUE;
    }

    if ($account->id() == $user->id()) {
      return TRUE;
    }

    if ($user->id() == 1) {
      return FALSE;
    }
    $account_weight = $this->getAccountRoleWeight($account);
    $user_weight = $this->getUserRoleWeight($user);
    return $this->hasRoleEditAccess($account_weight, $user_weight);
  }

}
